import ConsumerRepository from '../Repositories/ConsumerRepository.js'
import { log } from './LogController.js'
import md5 from 'md5'
import jwt from 'jsonwebtoken'

const getConsumerDetails = async ({ consDetails }) => {
    let response = consDetails
    let sonsNotifValues = []
    let childrenPhones = []
    
    if (consDetails.CDCLIENTE && consDetails.NRCPFRESPCON) {
        sonsNotifValues = await getSonsNotifValByParent(consDetails.CDCLIENTE, consDetails.NRCPFRESPCON)
        childrenPhones = await getChildrenPhones(consDetails.CDCLIENTE, consDetails.NRCPFRESPCON)
    }

    response.sonsNotifValues = sonsNotifValues
    response.childrenPhones = childrenPhones
    return response
}

const getSonsNotifValByParent = async (CDCLIENTE, NRCPFRESPCON) => {
    let sonsNotifValues = []
    sonsNotifValues = await ConsumerRepository.getSonsNotifValByParent(CDCLIENTE, NRCPFRESPCON)

    // Se for preciso parar de igualar os valores de notificação dos filhos, basta remover a linha abaixo
    sonsNotifValues = await matchNotifValues(sonsNotifValues, CDCLIENTE, NRCPFRESPCON)

    return sonsNotifValues
}

const matchNotifValues = async (sonsNotifValues, CDCLIENTE, NRCPFRESPCON) => {
    let biggestNotifValue = 0
    let affectedRows = 0

    for (let i = 0; i < sonsNotifValues.length; i++) {
        if (sonsNotifValues[i].VRNOTIFSALDO > biggestNotifValue) {
            biggestNotifValue = sonsNotifValues[i].VRNOTIFSALDO
        }
    }

    for (let i = 0; i < sonsNotifValues.length; i++) {
        if (biggestNotifValue > 0 && sonsNotifValues[i].VRNOTIFSALDO != biggestNotifValue) {
            let result = await ConsumerRepository.updateSonNotifVal(biggestNotifValue, CDCLIENTE, sonsNotifValues[i].CDCONSUMIDOR) 
            if (result?.rowsAffected > 0) affectedRows += result.rowsAffected
        }
    }

    if (affectedRows > 0) {
        sonsNotifValues = await ConsumerRepository.getSonsNotifValByParent(CDCLIENTE, NRCPFRESPCON)
    }

    return sonsNotifValues
}

const getChildrenPhones = async (CDCLIENTE, NRCPFRESPCON) => {
    let childrenPhones = []

    try {
        childrenPhones = await ConsumerRepository.getChildrenPhonesOriginal(CDCLIENTE, NRCPFRESPCON)
    } catch (error) {
        childrenPhones = await ConsumerRepository.getChildrenPhonesAlternative(CDCLIENTE, NRCPFRESPCON)
    }

    return childrenPhones
}

const updateProfile = async ({ consDetails, body, req }) => {
    let returnObj = {}
    let result
    let childPhonesUpdated = 0
    let sonNotifValuesUpdated = 0

    result = await ConsumerRepository.updateConsumerDetails(consDetails, body)
    childPhonesUpdated = await updateChildrenPhonesIfNeeded(consDetails, body)
    sonNotifValuesUpdated = await updateSonNotifValuesIfNeeded(consDetails, body)

    if (result?.rowsAffected > 0 || childPhonesUpdated > 0 || sonNotifValuesUpdated > 0) {
        let newConsDetails = await attConsDetails(consDetails)
        returnObj[process.env.TOKEN_HEADER_KEY] = jwt.sign({ CDFILIAL: consDetails.CDFILIAL, ...newConsDetails }, process.env.JWT_SECRET_KEY)
        returnObj.consDetails = await getConsumerDetails({ consDetails: newConsDetails })
    }

    return returnObj
}

const updateChildrenPhonesIfNeeded = async (consDetails, body) => {
    let affectedRows = 0
    let oldPhones = await getChildrenPhones(consDetails.CDCLIENTE, consDetails.NRCPFRESPCON) || []
    let newPhones = body.childrenPhones || []

    for (let i = 0; i < oldPhones.length; i++) {
        let oldPhone = oldPhones[i]
        let newPhone = newPhones.find(element => element.CDCONSUMIDOR === oldPhone.CDCONSUMIDOR)

        if (oldPhone.PHONE !== newPhone.PHONE) {
            let result

            try {
                result = await ConsumerRepository.updateChildrenPhonesOriginal(
                    consDetails.CDCLIENTE,
                    consDetails.NRCPFRESPCON,
                    newPhone.PHONE,
                    oldPhone.CDCONSUMIDOR
                )
            } catch (error) {
                result = await ConsumerRepository.updateChildrenPhonesAlternative(
                    consDetails.CDCLIENTE,
                    consDetails.NRCPFRESPCON,
                    newPhone.PHONE,
                    oldPhone.CDCONSUMIDOR
                )
            }

            if (result?.rowsAffected > 0) affectedRows += result.rowsAffected
        }
    }

    return affectedRows
}

const updateSonNotifValuesIfNeeded = async (consDetails, body) => {
    let affectedRows = 0
    let newNotifValue = body.sonNotifVal
    let oldNotifValues = await getSonsNotifValByParent(consDetails.CDCLIENTE, consDetails.NRCPFRESPCON)

    for (let i = 0; i < oldNotifValues.length; i++) {
        if (oldNotifValues[i].VRNOTIFSALDO !== newNotifValue) {
            let result = await ConsumerRepository.updateSonNotifVal(newNotifValue, consDetails.CDCLIENTE, oldNotifValues[i].CDCONSUMIDOR) 
            if (result?.rowsAffected > 0) affectedRows += result.rowsAffected
        }
    }

    return affectedRows
}

const attConsDetails = async (oldConsDetails) => {
    let params = {
        CDCLIENTE: oldConsDetails.CDCLIENTE,
        CDCONSUMIDOR: oldConsDetails.CDCONSUMIDOR
    }

    let newConsDetails = await ConsumerRepository.getConsumerDetails(params)
    return newConsDetails
}

const changePassword = async ({ consDetails, currentPassword, newPassword }) => {
    const encryptedCurrentPassword = md5(`odhen${currentPassword}7833`)

    const password = (await ConsumerRepository.getPassword(consDetails))?.CDSENHACONSMD5
    
    if (password !== encryptedCurrentPassword) {
        return {
            error: true,
            message: 'Senha atual inválida.'
        }
    } else {
        const encryptedNewPassword = md5(`odhen${newPassword}7833`)
        const result = await ConsumerRepository.changePassword(consDetails, encryptedNewPassword)
        return result
    }
}

export default { getConsumerDetails, updateProfile, changePassword }
