import moment from 'moment'
import { createLogsFolders, writeLogFile, zipOlderLogFiles } from './FileController.js'

/**
 * @param {string} type: info, error, etc
 * @param {boolean} serverSide
 * @param {boolean} skipConsole
 * @param {string | Object} message
 * @param {Object} consDetails
 * 
 * IMPORTANTE: Se a mensagem for ou contiver um objeto de erro, tem que passar como error.message ou error.stack.
 * Nunca o objeto error inteiro, porque o stringify no objeto error inteiro está causando perda de dados
 */
const log = ({ type = '', serverSide = true, skipConsole = false, message = '', consDetails = null }) => {
    try {
        createLogsFolders()

        let logMessage = `${type.toUpperCase()} - ${moment().format('DD/MM/YYYY HH:mm:ss')} \n`
        logMessage += typeof message === 'string' ? message : stringFrom(message)
        logMessage += '\n'
        if (consDetails) logMessage += `CONS DETAILS: ${stringFrom(consDetails)} \n`
        logMessage += '\n'

        let client = consDetails?.NMLOGINCONS ? consDetails.NMLOGINCONS : null
        writeLog(logMessage, serverSide, client, skipConsole)

        zipOlderLogFiles()
    } catch (error) {
        console.log(error)
    }
}

/**
 * @param {string} type: info, error, etc
 * @param {boolean} serverSide
 * @param {boolean} skipConsole
 * @param {string} route
 * @param {string | Object} requestMsg
 * @param {string | Object} responseMsg 
 * @param {Object} consDetails
 * 
 * IMPORTANTE: Se a mensagem for ou contiver um objeto de erro, tem que passar como error.message ou error.stack.
 * Nunca o objeto error inteiro, porque o stringify no objeto error inteiro está causando perda de dados
 */
const logCommunication = ({ type = '', serverSide = true, skipConsole = false, route = '', requestMsg = '', responseMsg = '', consDetails = null }) => {
    try {
        createLogsFolders()

        let logMessage = `${type.toUpperCase()} - ${moment().format('DD/MM/YYYY HH:mm:ss')} \n`
        logMessage += `ROUTE: ${route} \n`
        logMessage += `REQUEST: ${typeof requestMsg === 'string' ? requestMsg : stringFrom(requestMsg)} \n`
        logMessage += `RESPONSE: ${typeof responseMsg === 'string' ? responseMsg : stringFrom(responseMsg)} \n`
        if (consDetails) logMessage += `CONS DETAILS: ${stringFrom(consDetails)} \n`
        logMessage += '\n'

        let client = consDetails?.NMLOGINCONS ? consDetails.NMLOGINCONS : null
        writeLog(logMessage, serverSide, client, skipConsole)

        zipOlderLogFiles()
    } catch (error) {
        console.log(error)
    }
}

const stringFrom = (message) => {
    try {
        return JSON.stringify(message)
    } catch (error) {
        console.log(error.message)
        return ''
    }
}

const writeLog = (logMessage, serverSide, client, skipConsole) => {
    if (serverSide) {
        writeLogFile(logMessage)
    } else if (client) {
        writeLogFile(logMessage, client)
    }

    if (!skipConsole) console.log(logMessage)
}

export { log, logCommunication }
